@@ -13,6 +13,16 @@
 namespace clang {
 namespace find_all_symbols {
 
+HeaderMapCollector::HeaderMapCollector(
+    const RegexHeaderMap *RegexHeaderMappingTable) {
+  assert(RegexHeaderMappingTable);
+  this->RegexHeaderMappingTable.reserve(RegexHeaderMappingTable->size());
+  for (const auto &Entry : *RegexHeaderMappingTable) {
+    this->RegexHeaderMappingTable.emplace_back(llvm::Regex(Entry.first),
+                                               Entry.second);
+  }
+}
+
 llvm::StringRef
 HeaderMapCollector::getMappedHeader(llvm::StringRef Header) const {
   auto Iter = HeaderMappingTable.find(Header);
@@ -20,11 +30,13 @@ HeaderMapCollector::getMappedHeader(llvm::StringRef Header) const {
     return Iter->second;
   // If there is no complete header name mapping for this header, check the
   // regex header mapping.
-  if (RegexHeaderMappingTable) {
-    for (const auto &Entry : *RegexHeaderMappingTable) {
-      if (llvm::Regex(Entry.first).match(Header))
-        return Entry.second;
-    }
+  for (auto &Entry : RegexHeaderMappingTable) {
+#ifndef NDEBUG
+    std::string Dummy;
+    assert(Entry.first.isValid(Dummy) && "Regex should never be invalid!");
+#endif
+    if (Entry.first.match(Header))
+      return Entry.second;
   }
   return Header;
 }
